@@ -1,17 +1,32 @@
 import time
 import logging
+import hashlib
 
-from django.core.cache import cache
+from django.core.cache import get_cache, DEFAULT_CACHE_ALIAS
 from django.conf import settings
+import six
 
 from cacheback import tasks
 
-logging.basicConfig()
 logger = logging.getLogger('cacheback')
 
 MEMCACHE_MAX_EXPIRATION = 2592000
 
 
+def to_bytestring(value):
+    """
+    Encode a string as a UTF8 bytestring.  This function could be passed a
+    bytestring or unicode string so must distinguish between the two.
+    """
+    if isinstance(value, six.text_type):
+        return string.encode('utf8')
+    if isinstance(value, six.binary_type):
+        return value
+    if six.PY2:
+        return str(value)
+    return bytes(str(value), 'utf8')
+
+
 class Job(object):
     """
     A cached read job.
@@ -51,6 +66,10 @@ class Job(object):
     #: Overrides options for `refresh_cache.apply_async` (e.g. `queue`).
     task_options = {}
 
+    def __init__(self):
+        self.cache_alias = getattr(settings, 'CACHEBACK_CACHE_ALIAS', DEFAULT_CACHE_ALIAS)
+        self.cache = get_cache(self.cache_alias)
+
     # --------
     # MAIN API
     # --------
@@ -68,7 +87,7 @@ class Job(object):
 
         # Build the cache key and attempt to fetch the cached item
         key = self.key(*args, **kwargs)
-        item = cache.get(key)
+        item = self.cache.get(key)
 
         if item is None:
             # Cache MISS - we can either:
@@ -131,7 +150,7 @@ class Job(object):
         args = self.prepare_args(*raw_args)
         kwargs = self.prepare_kwargs(**raw_kwargs)
         key = self.key(*args, **kwargs)
-        item = cache.get(key)
+        item = self.cache.get(key)
         if item is not None:
             expiry, data = item
             self.cache_set(key, self.timeout(*args, **kwargs), data)
@@ -144,9 +163,9 @@ class Job(object):
         args = self.prepare_args(*raw_args)
         kwargs = self.prepare_kwargs(**raw_kwargs)
         key = self.key(*args, **kwargs)
-        item = cache.get(key)
+        item = self.cache.get(key)
         if item is not None:
-            cache.delete(key)
+            self.cache.delete(key)
 
     # --------------
     # HELPER METHODS
@@ -166,13 +185,13 @@ class Job(object):
         :expiry: The expiry timestamp after which the result is stale
         :data: The data to cache
         """
-        cache.set(key, (expiry, data), self.cache_ttl)
+        self.cache.set(key, (expiry, data), self.cache_ttl)
 
         if getattr(settings, 'CACHEBACK_VERIFY_CACHE_WRITE', True):
             # We verify that the item was cached correctly.  This is to avoid a
             # Memcache problem where some values aren't cached correctly
             # without warning.
-            __, cached_data = cache.get(key, (None, None))
+            __, cached_data = self.cache.get(key, (None, None))
             if data is not None and cached_data is None:
                 raise RuntimeError(
                     "Unable to save data of type %s to cache" % (
@@ -211,14 +230,13 @@ class Job(object):
             # refused.  When this happens, we try to run the task
             # synchronously.
             logger.error("Unable to trigger task asynchronously - failing "
-                         "over to synchronous refresh")
-            logger.exception(e)
+                         "over to synchronous refresh", exc_info=True)
             try:
                 return self.refresh(*args, **kwargs)
             except Exception as e:
                 # Something went wrong while running the task
-                logger.error("Unable to refresh data synchronously: %s", e)
-                logger.exception(e)
+                logger.error("Unable to refresh data synchronously: %s", e,
+                             exc_info=True)
             else:
                 logger.debug("Failover synchronous refresh completed successfully")
 
@@ -294,20 +312,28 @@ class Job(object):
             return self.class_path
         try:
             if args and not kwargs:
-                return "%s:%s" % (self.class_path, hash(args))
+                return "%s:%s" % (self.class_path, self.hash(args))
             # The line might break if your passed values are un-hashable.  If
             # it does, you need to override this method and implement your own
             # key algorithm.
             return "%s:%s:%s:%s" % (self.class_path,
-                                    hash(args),
-                                    hash(tuple(kwargs.keys())),
-                                    hash(tuple(kwargs.values())))
+                                    self.hash(args),
+                                    self.hash(tuple(kwargs.keys())),
+                                    self.hash(tuple(kwargs.values())))
         except TypeError:
             raise RuntimeError(
                 "Unable to generate cache key due to unhashable"
                 "args or kwargs - you need to implement your own"
                 "key generation method to avoid this problem")
 
+    def hash(self, value):
+        """
+        Generate a hash of the given tuple.
+
+        This is for use in a cache key.
+        """
+        return hashlib.md5(to_bytestring(value)).hexdigest()
+
     def fetch(self, *args, **kwargs):
         """
         Return the data for this job - this is where the expensive work should
